@@ -10,18 +10,18 @@
 #' @param mukeys vector of map unit keys
 #' @param WHERE character containing SQL WHERE clause specified in terms of fields in `legend`, `mapunit`, `component` or `coecosite` tables, used in lieu of `mukeys` or `areasymbols`
 #' @param query_string Default: `FALSE`; if `TRUE` return a character string containing query that would be sent to SDA via `SDA_query`
-#' @param ecoclasstypename If `NULL` no constraint on `ecoclasstypename` is used in the query.
+#' @param ecoclasstypename Default: `c("NRCS Rangeland Site", "NRCS Forestland Site")`. If `NULL` no constraint on `ecoclasstypename` is used in the query.
 #' @param ecoclassref Default: `"Ecological Site Description Database"`. If `NULL` no constraint on `ecoclassref` is used in the query.
 #' @param not_rated_value Default: `"Not assigned"`
 #' @param miscellaneous_areas logical. Include miscellaneous areas (non-soil components)?
 #' @param include_minors logical. Include minor components? Default: `TRUE`.
-#' @param threshold integer. Default: `0`. Minimum combined component percentage (RV) for inclusion of a mapunit's ecological site in wide-format tabular sumamry. Used only for `method="all"`.
+#' @param threshold integer. Default: `0`. Minimum combined component percentage (RV) for inclusion of a mapunit's ecological site in wide-format tabular summary. Used only for `method="all"`.
 #' @param dsn Path to local SQLite database or a DBIConnection object. If `NULL` (default) use Soil Data Access API via `SDA_query()`.
 #' @export 
 get_SDA_coecoclass <- function(method = "None",
                                areasymbols = NULL, mukeys = NULL, WHERE = NULL,
                                query_string = FALSE, 
-                               ecoclasstypename = NULL,
+                               ecoclasstypename = c("NRCS Rangeland Site", "NRCS Forestland Site"),
                                ecoclassref = "Ecological Site Description Database",
                                not_rated_value = "Not assigned",
                                miscellaneous_areas = TRUE,
@@ -148,7 +148,7 @@ get_SDA_coecoclass <- function(method = "None",
 
 .get_SDA_coecoclass_agg <- function(areasymbols = NULL,
                                     mukeys = NULL,
-                                    ecoclasstypename = NULL,
+                                    ecoclasstypename = c("NRCS Rangeland Site", "NRCS Forestland Site"),
                                     ecoclassref = "Ecological Site Description Database",
                                     not_rated_value = "Not assigned",
                                     miscellaneous_areas = TRUE,
@@ -156,7 +156,7 @@ get_SDA_coecoclass <- function(method = "None",
                                     dsn = NULL,
                                     threshold = 0) {
                                       
-  comppct_r <- NULL; condpct_r <- NULL; compname <- NULL; ecoclasstypename <- NULL
+  comppct_r <- NULL; condpct_r <- NULL; compname <- NULL; # ecoclasstypename <- NULL
   areasymbol <- NULL; compnames <- NULL; unassigned <- NULL;
   mukey <- NULL; .N <- NULL; .SD <- NULL; .GRP <- NULL;
   
@@ -199,28 +199,28 @@ get_SDA_coecoclass <- function(method = "None",
   res2 <- data.table::data.table(subset(res1, areasymbol != "US"))
   
   # remove FSG etc. some components have no ES assigned, but have other eco class
-  idx <- !res2$ecoclassref %in% c(not_rated_value, "Not assigned", "Ecological Site Description Database") &
-    !res2$ecoclasstypename %in% c(not_rated_value, "Not assigned", "NRCS Rangeland Site", "NRCS Forestland Site")
+  idx <- !res2$ecoclassref %in% c(not_rated_value, "Not assigned", ecoclassref) &
+    !res2$ecoclasstypename %in% c(not_rated_value, "Not assigned", ecoclasstypename)
   
   res2$ecoclassid[idx] <- not_rated_value
   res2$ecoclassref[idx] <- not_rated_value
   res2$ecoclassname[idx] <- not_rated_value
   res2$ecoclasstypename[idx] <- not_rated_value
   
+  .ECOCLASSTYPENAME <- ecoclasstypename
+  
   res3 <- res2[, list(
     condpct_r = sum(comppct_r, na.rm = TRUE),
-    compnames = paste0(compname[ecoclasstypename %in% c("NRCS Rangeland Site", 
-                                                        "NRCS Forestland Site")],
+    compnames = paste0(compname[ecoclasstypename %in% .ECOCLASSTYPENAME],
                        collapse = ", "),
-    unassigned = paste0(compname[!ecoclasstypename %in% c("NRCS Rangeland Site", 
-                                                          "NRCS Forestland Site")],
+    unassigned = paste0(compname[!ecoclasstypename %in% .ECOCLASSTYPENAME],
                         collapse = ", ")
   ), by = c("mukey", "ecoclassid", "ecoclassname")][, rbind(
     .SD[, 1:4],
     data.frame(
       ecoclassid = not_rated_value,
       ecoclassname = not_rated_value,
-      condpct_r = 100 - sum(condpct_r[nchar(compnames) > 0], na.rm = TRUE),
+      condpct_r = sum(condpct_r[nchar(unassigned) > 0 & !unassigned %in% compnames], na.rm = TRUE),
       compnames = paste0(unassigned[nchar(unassigned) > 0 & !unassigned %in% compnames],
                          collapse = ",")
     )
@@ -250,8 +250,8 @@ get_SDA_coecoclass <- function(method = "None",
     for (i in sdx) {
       if (i > nrow(x)) {
         d <- data.frame(
-          siten = NA_character_,
-          sitenname = NA_character_,
+          siten = not_rated_value,
+          sitenname = not_rated_value,
           sitencompname = NA_character_,
           sitenpct_r = NA_integer_,
           sitenlink = NA_character_
@@ -261,7 +261,7 @@ get_SDA_coecoclass <- function(method = "None",
           siten = ifelse(isTRUE(is.na(x$ecoclassid[i])), not_rated_value, x$ecoclassid[i]),
           sitenname = ifelse(isTRUE(is.na(x$ecoclassname[i])), not_rated_value, x$ecoclassname[i]),
           sitencompname = ifelse(isTRUE(is.na(x$compnames[i])), NA_character_, x$compnames[i]),
-          sitenpct_r = ifelse(isTRUE(is.na(x$condpct_r[i])), 0, x$condpct_r[i]),
+          sitenpct_r = ifelse(isTRUE(is.na(x$condpct_r[i])), 0L, x$condpct_r[i]),
           sitenlink = ifelse(
             isTRUE(x$ecoclassid[i] == not_rated_value | is.na(x$ecoclassid[i])),
             NA_character_,
